clear variables; clc;
close all hidden;

%%

% Notes:
% Clustering groups
% - Experienced vs inexperienced?
% - Effect on significance
% Load page time
% Need to double check ranking method (some can be equal rank)
% Need to look at methods 1-3 vs 4-6 (naively throwing away 7)

%% Settings

% Results files directory
results_dir = '../Subjective_test/Results';

% Number of test pages
nPages = 9;

% Number of sliders
nSliders = 7;

% Stimulus and attribute order to obtain
stim = kron((1:3),ones(1,3));
attr = repmat((1:3),[1,3]);

experience = min([7 4.5 5 inf 4 4 5 5 6 4 0 30 2.5 10 2 0 20 1 7 30 2 3],10);

%% Get raw results data

% Get file details
files = dir([results_dir filesep 'Part*']);
nFiles = length(files);

% Get list of available participants
participants = zeros(1,nFiles);
for i = 1:length(files)
    parti = regexp(files(i).name,'Part[0-9]+','match');
    parti = str2double(parti{1}(end+(-1:0)));
    participants(i) = parti;
end
participants = unique(participants);
nParticipants = length(participants);

% Get results data
pageOrder = zeros(nPages,nParticipants);
sliderOrder = zeros(nSliders,nParticipants);

% Number of ratings per paticipant
nPerPage = nParticipants*nSliders;
nRatings = nPerPage*nPages;

% To store all data in as participant, stimulus, attribute, method, rating,
% zscore, slider, page, page time
allData = zeros(nRatings,9);

% Index for storing
ind = 1:nSliders;
for i = 1:nParticipants
    % Get results data
    for j = 1:nPages
        % File to read
        fileij = dir(sprintf('%s%sPart%02iStim%02iAttr%02i*',results_dir,filesep, ...
            participants(i),stim(j),attr(j)));
        if length(fileij) ~= 1
            warning(['Incorrect number of results files (%i) for ' ...
                'participant %i'],length(fileij),participants(i));
        end
        fileij = [fileij.folder filesep fileij.name];
        % Store participant number
        allData(ind,1) = participants(i);
        % Store page
        pageij = regexp(fileij,'Page[0-9]+','match');
        pageij = str2double(pageij{1}(end+(-1:0)));
        allData(ind,8) = pageij;
        % Store stimulus and attribute numbers
        allData(ind,2) = stim(j);
        allData(ind,3) = attr(j);
        % Open file and get data
        fid = fopen(fileij);
        resj = fread(fid,'*char');
        resj(resj==';') = [];
        resj = cell2mat(textscan(resj,'%f %f','delimiter',','));
        fclose(fid);
        % Store method and rating
        allData(ind,4) = resj(2:end,1);
        allData(ind,5) = resj(2:end,2);
        % Store z score
        allData(ind,6) = (allData(ind,5)-mean(allData(ind,5)))./std(allData(ind,5));
        % Store slider order
        allData(ind,7) = 1:nSliders;
        % Store page time
        allData(ind,9) = resj(1,2);
        % Move along array for next time
        ind = ind+nSliders;
    end
end

% Sort by participant, stimulus, attribute, then method
allData = sortrows(allData,(1:4));

%%

% return
% 
% %% Write out results
% 
% csvwrite('allData.csv',allData);

%% Clustering

% Number of test combinations and participants
ind = true(size(allData,1),1);            % All answers
% ind = allData(:,2)==1 | allData(:,2)==2;    % Trajectories only
% ind = ind & allData(:,3)==1;              % And only location data

T = sum(ind)/nParticipants;

% Should be able to take first participants values as already sorted into
% same order
value = reshape(allData(ind,5),[T nParticipants]);

% If scaling values
value = (value-mean(value,2)*ones(1,nParticipants))./ ...
    (std(value,[],2)*ones(1,nParticipants));

% Perform clustering based on correlation coefficients
corrV = corr(value);
lowertri = (tril(ones(nParticipants))-diag(ones(1,nParticipants)))==1;
pdistV = sqrt(2*(1-corrV(lowertri).'));

% % Perform clustering based on Euclidean distances
% pdistV = pdist(value.');

figure
ZV = linkage(pdistV,'ward');
dendrogram(ZV)
xticklabels = participants(str2num(get(gca,'xticklabel')));
set(gca,'xticklabel',xticklabels)

% Cluster indices
clust = cluster(ZV,'maxclust',2);

%% Get participant subset

subgroup = participants;
% subgroup = participants(clust==1);
% subgroup = participants(clust==2);
% subgroup = participants(experience>=5);

% Number of participants to look at
nSubgroup = length(subgroup);

% Copy of all data with participants in cluster
[~,ind] = intersect(allData(:,1),subgroup);
someData = allData(kron(ind,ones(nSliders*nPages,1))+ ...
    repmat((0:nSliders*nPages-1).',[nSubgroup,1]),:);

% Experience (number of previous listening tests)
meanExperience = mean(experience);
[~,ind] = intersect(participants,subgroup);
groupExperience = experience(ind);
meanGroupExperience = mean(groupExperience);

%% Get ranked data

% NOTE: Careful with joint ranking!!!!

sortedData = sortrows(someData,[1 2 3 5],'descend');
rankData = zeros([nSliders,nSliders,nPages]);

ij = 1;
for i = 1:3 % Loop round stimulus
    for j = 1:3 % Loop round attributes
        % Get subset and ranking of data
        subset = sortedData(:,2)==i & sortedData(:,3)==j;
        sortedDataSubset = reshape(sortedData(subset,4),[nSliders,nSubgroup]);
        for k = 1:nSliders
            rankData(k,:,ij) = histcounts(sortedDataSubset(k,:),(0:nSliders)+0.5);
        end
        % Plot result
        figure
        bar(1:nSliders,flipud(rankData(:,:,ij)).',1,'stacked')
        xlim([0 nSliders]+0.5)
        ylim([0 nSubgroup])
        % Update index
        ij = ij+1;
    end
end

%% Basic box plots

ratingColumn = 5;   % raw data
% ratingColumn = 6;   % Z score

% data = allData(:,ratingColumn);
% X = nPage;
data = someData(someData(:,2)==1,ratingColumn);
X = 3;
data = reshape(data,[nSliders,X,nSubgroup]);
data = permute(data,[3 2 1]);
figure
[~,h] = box_plot(data,'limit','1.5IQR');

data = someData(someData(:,2)==2,ratingColumn);
X = 3;
data = reshape(data,[nSliders,X,nSubgroup]);
data = permute(data,[3 2 1]);
figure
[~,h] = box_plot(data,'limit','1.5IQR');

data = someData(someData(:,2)==3,ratingColumn);
X = 3;
data = reshape(data,[nSliders,X,nSubgroup]);
data = permute(data,[3 2 1]);
figure
[~,h] = box_plot(data,'limit','1.5IQR');

data = someData(someData(:,2)==1|someData(:,2)==2,ratingColumn);
X = 6;
data = reshape(data,[nSliders,X,nSubgroup]);
data = permute(data,[3 2 1]);
data = reshape(data,[nSubgroup*2,X/2,nSliders]);
figure
[~,h] = box_plot(data,'limit','1.5IQR');

%% For paper

fontSize = 10;
labels = {'VBAP1' 'VBIP1' 'Dual1' 'VBAP2' 'VBIP2' 'Dual2' 'Amb.'};
figure
for i = 1:3
    for j = 1:3
        ij = (i-1)*3+j;
        subplot(3,3,ij)
        data = someData(someData(:,2)==j&someData(:,3)==i,ratingColumn);
        data = reshape(data,[nSliders,nSubgroup]).';
        [~,h] = box_plot(data,'limit','1.5IQR');
        if i == 3
            set(gca,'xticklabels',labels)
            set(gca,'xticklabelrotation',90)
        else
            set(gca,'xticklabels',{''})
        end
        set(gca,'fontsize',fontSize)
        ylim([-5 105])
        box on
        if j == 1
            switch i
                case 1
                    ylabel(['Direction/movement';'    Smoothness    '],'fontsize',fontSize)
                    title('Trajectory 1','fontsize',fontSize)
                case 2
                    ylabel(['  Extent   ';'Consistency'],'fontsize',fontSize)
                    
                case 3
                    ylabel(['Spectral balance';'  Consistency   '],'fontsize',fontSize)
            end
        elseif j == 2
            switch i
                case 1
                    ylabel('Smoothness','fontsize',fontSize)
                    title('Trajectory 2','fontsize',fontSize)
                case 2
                    ylabel('Consistency','fontsize',fontSize)
                case 3
                    ylabel('Consistency','fontsize',fontSize)
            end
        elseif j == 3
            switch i
                case 1
                    ylabel('Simmilarity','fontsize',fontSize)
                    title('Static','fontsize',fontSize)
                case 2
                    ylabel('Simmilarity','fontsize',fontSize)
                case 3
                    ylabel('Simmilarity','fontsize',fontSize)
            end
        end
    end
end

% figure
% for i = 1:3
%     subplot(3,1,i)
%     data = someData((someData(:,2)==1|someData(:,2)==2)&someData(:,3)==i,ratingColumn);
%     data = reshape(data,[nSliders,2,nSubgroup]);
%     data = permute(data,[3 1 2]);
%     [~,h] = box_plot(data,'limit','1.5IQR');
% end

%% ANOVA

title1 = {'Location','Extent','Spectral balance'};

% Trajectories...

for i = 1:3
    
    % Perform ANOVA
    subset = (someData(:,2)==1|someData(:,2)==2) & someData(:,3)==i;
    participantsList = someData(subset,1);
    TrajectoryList = someData(subset,2);
    RendererList = someData(subset,4);
    ratingsList = someData(subset,5);
    [pi,ti,stats,~] = anovan(ratingsList, ...
        {participantsList TrajectoryList RendererList}, ...
        'model','interaction', ...
        'varnames',{'Participant','Trajectory','Renderer'}, ...
        'random',1, ...
        'display','on');
    
    % Compare individual dependent variable comparisons
    figure; compare1 = multcompare(stats,'Dimension',1); title(['Trajectory: ' title1{i}])
    figure; compare2 = multcompare(stats,'Dimension',2); title(['Trajectory: ' title1{i}])
    figure; compare3 = multcompare(stats,'Dimension',3); title(['Trajectory: ' title1{i}])

end

% Static source...

for i = 1:3
    
    % Perform ANOVA
    subset = someData(:,2)==3 & someData(:,3)==i;
    participantsList = someData(subset,1);
    RendererList = someData(subset,4);
    ratingsList = someData(subset,5);
    [pi,ti,stats,~] = anovan(ratingsList, ...
        {participantsList RendererList}, ...
        'varnames',{'Participant','Renderer'}, ...
        'random',1, ...
        'display','on');

    % Compare individual dependent variable comparisons
%     figure; compare1 = multcompare(stats,'Dimension',1); title(['Static: ' title1{i}])
    figure; compare2 = multcompare(stats,'Dimension',2); title(['Static: ' title1{i}])

end

%% ANOVA separate trajectories

% Trajectory 1...

for i = 1:3
    
    % Perform ANOVA
    subset = someData(:,2)==1 & someData(:,3)==i;
    participantsList = someData(subset,1);
    RendererList = someData(subset,4);
    ratingsList = someData(subset,5);
    [pi,ti,stats,~] = anovan(ratingsList, ...
        {participantsList RendererList}, ...
        'varnames',{'Participant','Renderer'}, ...
        'random',1, ...
        'display','on');

    % Compare individual dependent variable comparisons
%     figure; compare1 = multcompare(stats,'Dimension',1); title(['Trajectory 1: ' title1{i}])
    figure; compare2 = multcompare(stats,'Dimension',2); title(['Trajectory 1: ' title1{i}])

end

% Trajectory 2...

for i = 1:3
    
    % Perform ANOVA
    subset = someData(:,2)==2 & someData(:,3)==i;
    participantsList = someData(subset,1);
    RendererList = someData(subset,4);
    ratingsList = someData(subset,5);
    [pi,ti,stats,~] = anovan(ratingsList, ...
        {participantsList RendererList}, ...
        'varnames',{'Participant','Renderer'}, ...
        'random',1, ...
        'display','on');

    % Compare individual dependent variable comparisons
%     figure; compare1 = multcompare(stats,'Dimension',1); title(['Trajectory 2: ' title1{i}])
    figure; compare2 = multcompare(stats,'Dimension',2); title(['Trajectory 2: ' title1{i}])

end

%% ANOVA separate algorithms (VBAP vs VBIP vs dual; L1 vs L2)

% Both trajectories...

for i = 1:3
    
    % Perform ANOVA
    subset = (someData(:,2)==1|someData(:,2)==2) & ...
        someData(:,3)==i & someData(:,4)~=7;
    participantsList = someData(subset,1);
    TrajectoryList = someData(subset,2);
    panningList = someData(subset,4);
    normList = panningList;
    normList(normList<=3) = 1;  normList(normList>=4) = 2;
    panningList = mod(panningList-1,3)+1;
    ratingsList = someData(subset,5);
    [pi,ti,stats,~] = anovan(ratingsList, ...
        {participantsList TrajectoryList panningList normList}, ...
        'model','interaction', ...
        'varnames',{'Participant','Trajectory','Panning','Norm'}, ...
        'random',1, ...
        'display','on');

   % Compare individual dependent variable comparisons
    figure; compare1 = multcompare(stats,'Dimension',1); title(['Trajectory: ' title1{i}])
    figure; compare2 = multcompare(stats,'Dimension',2); title(['Trajectory: ' title1{i}])
    figure; compare3 = multcompare(stats,'Dimension',3); title(['Trajectory: ' title1{i}])
    figure; compare4 = multcompare(stats,'Dimension',4); title(['Trajectory: ' title1{i}])

end

% Trajectory 1...

for i = 1:3
    
    % Perform ANOVA
    subset = someData(:,2)==1 & ...
        someData(:,3)==i & someData(:,4)~=7;
    participantsList = someData(subset,1);
    TrajectoryList = someData(subset,2);
    panningList = someData(subset,4);
    normList = panningList;
    normList(normList<=3) = 1;  normList(normList>=4) = 2;
    panningList = mod(panningList-1,3)+1;
    ratingsList = someData(subset,5);
    [pi,ti,stats,~] = anovan(ratingsList, ...
        {participantsList panningList normList}, ...
        'model','interaction', ...
        'varnames',{'Participant','Panning','Norm'}, ...
        'random',1, ...
        'display','on');

   % Compare individual dependent variable comparisons
    figure; compare1 = multcompare(stats,'Dimension',1); title(['Trajectory 1: ' title1{i}])
    figure; compare2 = multcompare(stats,'Dimension',2); title(['Trajectory 1: ' title1{i}])
    figure; compare3 = multcompare(stats,'Dimension',3); title(['Trajectory 1: ' title1{i}])

end

for i = 1:3
    
    % Perform ANOVA
    subset = someData(:,2)==2 & ...
        someData(:,3)==i & someData(:,4)~=7;
    participantsList = someData(subset,1);
    TrajectoryList = someData(subset,2);
    panningList = someData(subset,4);
    normList = panningList;
    normList(normList<=3) = 1;  normList(normList>=4) = 2;
    panningList = mod(panningList-1,3)+1;
    ratingsList = someData(subset,5);
    [pi,ti,stats,~] = anovan(ratingsList, ...
        {participantsList panningList normList}, ...
        'model','interaction', ...
        'varnames',{'Participant','Panning','Norm'}, ...
        'random',1, ...
        'display','on');

   % Compare individual dependent variable comparisons
    figure; compare1 = multcompare(stats,'Dimension',1); title(['Trajectory 2: ' title1{i}])
    figure; compare2 = multcompare(stats,'Dimension',2); title(['Trajectory 2: ' title1{i}])
    figure; compare3 = multcompare(stats,'Dimension',3); title(['Trajectory 2: ' title1{i}])

end

