# ------------------------------------------------------------------------------
# 
# Do quick ANOVAs on data in this directory
#
# ------------------------------------------------------------------------------

# Clear stuff
rm(list=ls())

# Load ordinal package
library("ez")
library(plyr)

# Define path
setwd("~/S3A/aes2018_dualband_panning/Evaluation/")

data <- read.csv("allData.csv",header=FALSE)
colnames(data) <- c("PID", "STIM", "ATTR", "METH", "SCORE", "ZSCORE","SLIDER", "PAGE")

ids = c(1:max(data$PID,na.rm = FALSE)) # All
# ids = c(3,4,7,9,12:16,21,22)
data <- data[data$PID%in%ids,]

# Factors
data$PID <- factor(data$PID)
data$STIM <- factor(data$STIM)
data$ATTR <- factor(data$ATTR)
data$METH <- factor(data$METH)
data$SLIDER <- factor(data$SLIDER)
data$PAGE <- factor(data$PAGE)

# Label factors
data$STIM <- revalue(data$STIM, c("1" = "Trajectory 1", "2" = "Trajectory 2", "3" = "Static"))
data$ATTR <- revalue(data$ATTR, c("1" = "Direction", "2" = "Extent", "3"="Timbre"))
data$METH <- revalue(data$METH, c("1" = "VBAP", "2" = "VBIP","3"="Dual","4"="L2 VBAP","5"="L2 VBIP","6"="L2 Dual","7"="Ambisonics"))

# Subsets
dataT <- subset(data,STIM == "Trajectory 1" | STIM == "Trajectory 2")
dataT1 <- subset(dataT,STIM == "Trajectory 1")
dataT2 <- subset(dataT,STIM == "Trajectory 2")
dataS <- subset(data,STIM == "Static")

# Perform ANOVAs for trajectories
anTDir = aov(SCORE~METH*STIM+Error(PID/(METH*STIM)),data=subset(dataT,ATTR == "Direction"))
anTExt = aov(SCORE~METH*STIM+Error(PID/(METH*STIM)),data=subset(dataT,ATTR == "Extent"))
anTTim = aov(SCORE~METH*STIM+Error(PID/(METH*STIM)),data=subset(dataT,ATTR == "Timbre"))

# Print summaries
cat('\n\n2 way repeated ANOVA:\n\n'); summary(anTDir)
cat('\n\n2 way repeated ANOVA:\n\n'); summary(anTExt)
cat('\n\n2 way repeated ANOVA:\n\n'); summary(anTTim)

# Perform ANOVAs for static
anSDir = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataS,ATTR == "Direction"))
anSExt = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataS,ATTR == "Extent"))
anSTim = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataS,ATTR == "Timbre"))

# Print summaries
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anSDir)
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anSExt)
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anSTim)




# Perform ANOVAs for Trajectory 1
anT1Dir = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataT1,ATTR == "Direction"))
anT1Ext = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataT1,ATTR == "Extent"))
anT1Tim = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataT1,ATTR == "Timbre"))

# Print summaries
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anT1Dir)
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anT1Ext)
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anT1Tim)


# Perform ANOVAs for Trajectory 1
anT2Dir = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataT2,ATTR == "Direction"))
anT2Ext = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataT2,ATTR == "Extent"))
anT2Tim = aov(SCORE~METH+Error(PID/(METH)),data=subset(dataT2,ATTR == "Timbre"))

# Print summaries
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anT2Dir)
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anT2Ext)
cat('\n\n1 way repeated ANOVA:\n\n'); summary(anT2Tim)



# Pairwise comparisons

dataset = subset(dataT,ATTR == "Direction")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")
dataset = subset(dataT,ATTR == "Extent")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")
dataset = subset(dataT,ATTR == "Timbre")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")

dataset = subset(dataT1,ATTR == "Direction")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")
dataset = subset(dataT1,ATTR == "Extent")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")
dataset = subset(dataT1,ATTR == "Timbre")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")

dataset = subset(dataT2,ATTR == "Direction")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")
dataset = subset(dataT2,ATTR == "Extent")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")
dataset = subset(dataT2,ATTR == "Timbre")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")

dataset = subset(dataS,ATTR == "Direction")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")
dataset = subset(dataS,ATTR == "Extent")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")
dataset = subset(dataS,ATTR == "Timbre")
pairwise.t.test(dataset$SCORE,dataset$METH,p.adj="bonferroni")








