# -*- coding: utf-8 -*-
"""
Created on Sun Feb 26 16:41:01 2017

@author: andi
"""

import visr
import rcl
import pml

from allrad_gain_calculator import AllRadGainCalculator

class AllRadPanner( visr.CompositeComponent ):
    def __init__(self, context, name, parent, *, # Asterisk has no effect but to force all subsequent
             numberOfInputs,
             regularArrayConfig,
             realArrayConfig,
             interpolationSteps,
             hoaOrder,
             orderWeights = None ):
        super( AllRadPanner, self ).__init__(context, name, parent )

        numberOfOutputs = realArrayConfig.numberOfRegularLoudspeakers

        self.input = visr.AudioInputFloat( "in", self, numberOfInputs )
        self.output = visr.AudioOutputFloat( "out", self, numberOfOutputs )

        self.objectInput = visr.ParameterInput( "objects", self,
                                               parameterType=pml.ObjectVector.staticType,
                                               protocolType=pml.DoubleBufferingProtocol.staticType,
                                               parameterConfig=pml.EmptyParameterConfig() )

        # Computation of the decoder matrix
        self.calculator = AllRadGainCalculator( context, "GainCalculator", self,
                                               numberOfObjects = numberOfInputs,
                                               regularArrayConfig = regularArrayConfig,
                                               realArrayConfig = realArrayConfig,
                                               hoaOrder = hoaOrder,
                                               orderWeights = orderWeights )

        self.panningMatrix = rcl.GainMatrix( context, "PanningMatrix", self,
                                             numberOfInputs=numberOfInputs,
                                             numberOfOutputs=numberOfOutputs,
                                             interpolationSteps=interpolationSteps,
                                             initialGains = 0.0,
                                             controlInput=True )

        self.audioConnection( self.input, self.panningMatrix.audioPort("in") )
        self.audioConnection( self.panningMatrix.audioPort("out"), self.output )

        self.parameterConnection( self.objectInput, self.calculator.parameterPort("objectVectorInput") )
        self.parameterConnection( self.calculator.parameterPort("gainOutput"),
                                 self.panningMatrix.parameterPort("gainInput") )
