# -*- coding: utf-8 -*-
"""
Created on Thu Jan 18 09:43:35 2018

@author: af5u13
"""

import lxml.etree as ET

from scipy.spatial import ConvexHull
import numpy as np

# from operator import attrgetter

# Local definitions, consider moving to separate file.
def deg2rad( phi ):
    return (np.pi/180.0) * phi

def rad2deg( phi ):
    return (180.0/np.pi) * phi

def cart2sph( X ):
    x = X[...,0]
    y = X[...,1]
    z = X[...,2]
    radius = np.sqrt( x*x + y*y + z*z );
    az = np.arctan2( y, x );
    el = np.arcsin( z / radius );
    return az, el, radius


#TODO add EQ configuration support
#TODO Add subwoofer configuration support
def createArrayConfigFile( outputFileName,
                           lspPositions,
                           twoDconfig = False,
                           sphericalPositions=False,
                           channelIndices = None,
                           loudspeakerLabels = None,
                           triplets = None,
                           lspDelays = None,
                           lspGainDB = None,
                           virtualLoudspeakers = [],
                           ):
    if len( lspPositions.shape ) != 2 or (lspPositions.shape[0] not in [2,3]):
        raise ValueError( "The loudspeaker position data must be a 2D array with leading dimension 2 or 3" )

    numRealLoudspeakers = lspPositions.shape[-1]

    if lspPositions.shape[0] == 2:
        lspPositions = np.concatenate( (lspPositions, np.zeros( 1, numRealLoudspeakers )), axis = 0 )

    if channelIndices is None:
        channelIndices = [ i+1 for i in range(numRealLoudspeakers) ]
    else:
        if len( channelIndices ) != numRealLoudspeakers:
            raise ValueError( "If the 'channelIndices' argument is given, its length must match the number of real loudspeakers." )

    if loudspeakerLabels is None:
        # Create default numbering 1...numRealLoudspeakers
        loudspeakerLabels = [ "lsp%d" % (i+1) for i in range(numRealLoudspeakers) ]
    else:
        if len( loudspeakerLabels ) != numRealLoudspeakers:
            raise ValueError( "If the 'loudspeakerLabels' argument is given, its length must match the number of real loudspeakers." )

    if virtualLoudspeakers is None:
        numVirtualLoudspeakers = 0
        virtualLoudspeakerPos = np.zeros( (3,0), dtype=lspPositions.dtype )
        virtualLoudspeakerLabels = []
    else:
        numVirtualLoudspeakers = len( virtualLoudspeakers )
        virtualLoudspeakerLabels = [ record["id"] for record in virtualLoudspeakers ]
        virtualLoudspeakerPos = np.zeros( (3, numVirtualLoudspeakers), dtype=lspPositions.dtype )
        for (idx,record) in enumerate(virtualLoudspeakers):
            pos = np.asarray(record["pos"])
            virtualLoudspeakerPos[0:pos.shape[-1],idx] = pos

#        if virtualLoudspeakers.shape[0] == 2:
#            virtualLoudspeakers = np.concatenate( (virtualLoudspeakers, np.zeros( 1, numVirtualLoudspeakers )), axis = 0 )
#        if virtualLoudspeakerLabels is None:
#            virtualLoudspeakerLabels = [ "virtual%d" % (i+1) for i in range(numVirtualLoudspeakers) ]
#        else:
#            if len( virtualLoudspeakerLabels ) != numVirtualLoudspeakers:
#                raise ValueError( "If the 'loudspeakerLabels' argument is given, its length must match the number of real loudspeakers." )
    allLoudspeakers = np.concatenate( (lspPositions, virtualLoudspeakerPos ), axis=-1 )
    allLabels = loudspeakerLabels + virtualLoudspeakerLabels

    if triplets is None:
        hull = ConvexHull( allLoudspeakers.T )

        triplets = hull.simplices
#    else:
        # TODO: Perform checking of the triplets
#        for polygon in triplets:
#            labeledTriplets.append( [ allLabels[i] for i in polygon ] )
    labeledTriplets = [[ allLabels[i] for i in polygon ] for polygon in triplets ]

    # %% Create the document tree
    xmlRoot = ET.Element( "panningConfiguration" )
    for lspIdx in range(numRealLoudspeakers):
        lspNode = ET.SubElement( xmlRoot, "loudspeaker" )
        lspNode.set( "id", str(loudspeakerLabels[lspIdx]) )
        lspNode.set( "channel", str(channelIndices[lspIdx]) )
        if sphericalPositions:
            az, el, radius = cart2sph( lspPositions[:,lspIdx] )
            sphNode = ET.SubElement( lspNode, str("polar") )
            sphNode.set( "az", str(round(rad2deg( az ),10)) )
            if not twoDconfig:
                sphNode.set( "el",  str(round(rad2deg( el ),10)) )
            sphNode.set( "r", str(radius) )
        else:
            cartNode = ET.SubElement( lspNode, "cart" )
            cartNode.set( "x", str(round(lspPositions[0,lspIdx],10)) )
            cartNode.set( "y", str(round(lspPositions[1,lspIdx],10)) )
            if not twoDconfig:
                cartNode.set( "z", str(lspPositions[2,lspIdx]) )
        if lspDelays is not None:
            lspNode.set( "delay", str(round(lspDelays,10)) )
        if lspGainDB is not None:
            lspNode.set( "gainDB", str(round(lspGainDB,10)) )
        # TODO: Optional EQ config

    for (virtIdx, virtRecord) in enumerate( virtualLoudspeakers ):
        virtualLspNode = ET.SubElement( xmlRoot, "virtualspeaker" )
        virtualLspNode.set( "id", str(virtualLoudspeakerLabels[virtIdx]) )
        if sphericalPositions:
            az, el, radius = cart2sph( virtualLoudspeakerPos[:,virtIdx] )
            sphNode = ET.SubElement( virtualLspNode, "polar" )
            sphNode.set( "az", str(rad2deg( az )) )
            if not twoDconfig:
                sphNode.set( "el", str( rad2deg( el )) )
            sphNode.set( "r", str(radius) )
        else:
            cartNode = ET.SubElement( lspNode, "cart" )
            cartNode.set( "x", str(virtualLoudspeakerPos[0,virtIdx]) )
            cartNode.set( "y", str(virtualLoudspeakerPos[1,virtIdx]) )
            if not twoDconfig:
                cartNode.set( "z", str(virtualLoudspeakerPos[2,virtIdx]) )
        if "routing" in virtRecord:
            for routingEntry in virtRecord["routing"]:
                ( routingLabel, gain ) = routingEntry
                if routingLabel not in loudspeakerLabels:
                    raise ValueError( "Routing loudspeaker label %s does not match an existing (real) loudspeaker." % routingLabel )
                gainDB = 20.0*np.log10( gain )
                routingNode = ET.SubElement( virtualLspNode, "route" )
                routingNode.set( "lspId", str(routingLabel) )
                routingNode.set( "gainDB", str( round(gainDB,4) ) )


    for triplet in labeledTriplets:
        tripletNode = ET.SubElement( xmlRoot, "triplet" )
        # For the moment, we need triplets unless we have a 2d config.
        if (twoDconfig and len(triplet )!=2) or (not twoDconfig and len(triplet )!=3) :
            raise ValueError( "triplets must consist of eiter 3 vertices (£D setup) or two vertices (2D setup)" )
        tripletNode.set( "l1", str(triplet[0]) )
        tripletNode.set( "l2", str(triplet[1]) )
        if not twoDconfig:
            tripletNode.set( "l3", str(triplet[2]) )

    # %% Write the array configuration.
    eTree = ET.ElementTree( element = xmlRoot )
    eTree.write( outputFileName, pretty_print=True )