# -*- coding: utf-8 -*-
"""
Created on Thu Jan 18 10:21:11 2018

@author: af5u13
"""

# Add 'helper' folder to path
import sys, inspect, os
sys.path.append(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))+'/helper')

from createArrayConfigFile import createArrayConfigFile

from create_mpegh_downmix_config import createMpegHDownMix

import numpy as np

def sph2cart( az, el, r ):
    (azBC, elBC, rBC ) = np.broadcast_arrays( az, el, r )
    elFactor = np.cos( elBC )
    x = np.cos( azBC ) * elFactor * rBC
    y = np.sin( azBC ) * elFactor * rBC
    z = np.sin( elBC ) * rBC
    cart = np.stack( (x,y,z), -1 )
    return cart

def deg2rad( phi ):
    return (np.pi/180.0) * phi

def rad2deg( phi ):
    return (180.0/np.pi) * phi

def cart2sph( X ):
    x = X[...,0]
    y = X[...,1]
    z = X[...,2]
    radius = np.sqrt( x*x + y*y + z*z );
    az = np.arctan2( y, x );
    el = np.arcsin( z / radius );
    return az, el, radius



# 4+5+0 setup, one virtual loudspeaker
#az = deg2rad( np.asarray([0, -30, 30, -110, 110, -30, 30, -110, 110] ) )
#el = deg2rad( np.asarray([-0.1,   -0.1,  -0.1,    -0.1,   -0.1,  30, 30,   30,  30] ) )
#loudspeakerLabels = ["M+000", "M-030", "M+030", "M-110", "M+110", "U-030", "U+030", "U-110", "U+110"]
#VoSRouting = [("M+000", 0.2),("M+030", 0.2),("M-030", 0.2),("M+110", 0.2),("M-110", 0.2)]
#fileName = "../data/bs2051-4+5+0.xml"
#fileNameDownmix = "../data/bs2051-4+5+0_mpeg-h_downmix.xml"
#delays = None

# Salford audio booth setup (pseudo 4+5+0 setup)
az = deg2rad( np.asarray([0, 45, 135, -135, -45, 45, 135, -135, -45] ) )
el = deg2rad( np.asarray([-0.1,   -0.1,  -0.1,    -0.1,   -0.1,  30, 30,   30,  30] ) )
loudspeakerLabels = ["M+000", "M+045", "M+135", "M-135", "M-045", "U+045", "U+135", "U-135", "U-045"]
VoSRouting = [("M+000", 0.2),("M+045", 0.2),("M-045", 0.2),("M+135", 0.2),("M-135", 0.2)]
fileName = "../data/salfordAudioBooth-4+5+0.xml"
fileNameDownmix = "../data/salfordAudioBooth-4+5+0_mpeg-h_downmix.xml"
delays = None

lspData = sph2cart( az, el, 1 ).T

# virtSpeaker = np.asarray( [[0.0, 0.0, -1.0 ]] ).T
virtSpeaker = [{"id": "VoS", "pos": [0.0, 0.0, -1.0], "routing": VoSRouting }]

#loudspeakerLabels = ["M+000", "M-030", "M+030", "M-110", "M+110", "U-030", "U+030", "U-110", "U+110"]
# We need a very recent VISR build to have a bug with special chareacters (+-) in loudspeaker labels fixed.
# For the moment use conservative labels.
# loudspeakerLabels = [ str(i+1) for i in range(lspData.shape[-1]) ]


virtualLoudspeakerData, triplets  = createMpegHDownMix( lspData, loudspeakerLabels )
# virtualLoudspeakerLabels = ["VotD"]

# ISVR-specific channel indices
# Note: the comparison renderer does not currently understand
# channelIndices = [ 16, 17, 27, 20, 24, 29, 35, 31, 34]


createArrayConfigFile( fileNameDownmix,
                      lspPositions = lspData,
                      triplets=triplets,
                      twoDconfig = False,
                      sphericalPositions=True,
                      lspDelays = delays,
#                       channelIndices=channelIndices,
                      loudspeakerLabels = loudspeakerLabels,
                      virtualLoudspeakers = virtualLoudspeakerData
                      )

# Setup with the bare minimum virtual loudspeaker (VoS)
#virtSpeaker = [{"id": "VoS", "pos": [0.0, 0.0, -1.0], "routing": [("M+000", 0.2),("M+030", 0.2),("M-030", 0.2),
#                ("M+110", 0.2),("M-110", 0.2)] }]

createArrayConfigFile( fileName,
                      lspPositions = lspData,
                      twoDconfig = False,
                      sphericalPositions=True,
                      lspDelays = delays,
#                       channelIndices=channelIndices,
                      loudspeakerLabels = loudspeakerLabels,
                      virtualLoudspeakers = virtSpeaker
                      )

