# -*- coding: utf-8 -*-
"""
Created on Wed Jan 24 15:42:14 2018

@author: af5u13
"""

from vectorFunctions import normalise, rotateToXYPlane, edgeNormalVectors

from scipy.spatial import ConvexHull
import numpy as np

def triangleCentre( L ):
    rhs = np.ones( L.shape[0:-2]+(L.shape[-1],), dtype=L.dtype )
    cRaw = np.linalg.solve( L, rhs )
    cNorm = np.linalg.norm( cRaw, ord = 2, axis = -1 )
    cNormed = cRaw / cNorm[...,np.newaxis]
    return cNormed

#def triangleCentres( Lfull, Lindices ):
#    pass


def createMpegHDownMix( lspPos, lspLabels ):
    virtCnt = 0

    # numSpeakers = lspPos.shape[0]

    L = normalise( lspPos, axis = -1, norm=2 ).T

    hull = ConvexHull( L )

    # %% Start with an initial tesselation of cyclic polygons.
    # TODO: Move to library function.
    triplets = hull.simplices.tolist()
    tripletVertices = hull.points[hull.simplices]
    centres = triangleCentre( tripletVertices )

    distMtx = centres @ centres.T

    # Tolerance of dot products when searching for coplanarity.
    distEps = 1e-5 # TODO: Relate tolerance to angle

    nextVertexIdx = L.shape[0]

    allIndices = np.array( range(0,centres.shape[0]))

    virtualSpeakers = []

    for idx in range( 0, centres.shape[0] ):
        # Check for special marker value whether this triplet has already been processed.
        if np.isnan( distMtx[idx,0] ):
            continue

        newFacet = set( hull.simplices[idx] )

        matchMask = distMtx[idx,idx+1:] >= (1.0-distEps )

        matchIndices = allIndices[idx+1:][matchMask]

        # Mark the rows of the processed other triangles as processed.
        distMtx[matchIndices,:] = np.NAN

        for extraTriangleIdx in matchIndices:
            extraTriangle = set( hull.simplices[extraTriangleIdx,...] )
            newFacet = newFacet.union( extraTriangle )

        # %% Compute a nonintersecting ordering (counterclockwise
        # on the face of the plane)
        normalVec = centres[idx,:]

        facetIndices = np.asarray(list(newFacet)) # Create a ndarray if ondices from a set
        vertices = L[facetIndices,:]

        # %% Rotate the normal plane to the x-y plane
        # First, move all the vectors to the plane perpendicular to the centre vector
        projFac = np.dot( vertices, normalVec )
        verticesInPlane = vertices - projFac[:,np.newaxis] * normalVec[np.newaxis,:]
        planeVertices = rotateToXYPlane( verticesInPlane, normalVec )

        planeAngles = np.arctan2( planeVertices[...,1], planeVertices[...,0])
        sortIdcs = np.argsort( planeAngles )
        # planeVerticesSorted = planeVertices[ sortIdcs, : ]
        indicesSorted = facetIndices[sortIdcs]

        # points = L[indicesSorted,: ]

        centre = np.asarray(centres[idx])
        # Pick one of the vertices for computing the radius (results are equal for all vertices)

        radius = np.arccos( np.dot( centre, L[indicesSorted[0],:] ) )

        numSurrounding = len( indicesSorted )
        # TODO: Define more sophisticated limit for maxmum admitted radius.
        if (numSurrounding > 3) or (radius >= 0.5*np.pi - distEps):
            # Insert virtual loudspeaker

            virtLabel = "virt%d" % virtCnt
            virtCnt += 1

            currVertexIdx = nextVertexIdx

            # Remove all triangles covered by the current cyclic polygon
            removeIdcs = []
            currVerticesSet = set( indicesSorted )
            for idx, tri in enumerate( triplets ):
                if set( tri ).issubset( currVerticesSet ):
                    removeIdcs.append( idx )
            for idx in reversed( removeIdcs ):
                del triplets[idx]
            for vertexIdx in range( numSurrounding ):
                nextTriplet = [ indicesSorted[vertexIdx],
                               indicesSorted[(vertexIdx+1) % numSurrounding],
                               currVertexIdx ]
                triplets.append( nextTriplet )

            downmixGain = 1.0 / numSurrounding
            routingList = []
            for vertexIdx in range(numSurrounding):
                spkLabel = lspLabels[indicesSorted[vertexIdx]]
                routingList.append( (spkLabel, downmixGain ))
            virtRecord = { "id": virtLabel, "pos": centre.tolist(), "routing": routingList }
            virtualSpeakers.append( virtRecord )

            nextVertexIdx += 1

    return virtualSpeakers, triplets
