# -*- coding: utf-8 -*-
"""
Created on Thu Nov  2 13:09:18 2017

@author: af5u13
"""

import numpy as np

def normalise( B, norm=2, axis = -1 ):
    mag = np.linalg.norm( B, ord=norm, axis=axis )
    magExp = np.expand_dims( mag, axis )
    Bnorm = B / magExp
    return Bnorm

def angleDifference( vec1, vec2, axis=-1 ):
    dot= np.sum(normalise(vec1) * normalise(vec2), axis=axis)
    return np.arccos( np.clip(dot, None, 1.0) )


def rotateToXYPlane( x, normalVec ):
    phi = np.arctan2( normalVec[1], normalVec[0] )
    rz = np.asarray( [[np.cos(phi), -np.sin(phi), 0.0 ],
                      [np.sin(phi), np.cos(phi), 0.0 ],
                      [0., 0., 1.0 ]] )
    n2 = normalVec @ rz
    theta = - np.arctan2( n2[0], n2[2] ) # Rotation reversed
    ry = np.asarray( [[np.cos(theta), 0.0, -np.sin(theta) ],
                      [0.0, 1.0, 0.0],
                      [np.sin(theta), 0.0, np.cos(theta) ] ] )
    rotZY = rz @ ry
    y = x @ rotZY
    return y


def edgeNormalVectors( vertices ):
    """
    Compute the normal vectors for the edges of a spherical polygon specified by a number of points.
    Vertices are expected as a #vertices x #dimension 2D numpy.ndarray.
    """
    verticesShifted = np.roll( vertices, -1, axis=0 )
    edgePerpendiculars = np.cross( vertices, verticesShifted ) # All along last axis.
    return normalise( edgePerpendiculars, axis = -1 )
