# -*- coding: utf-8 -*-
"""
Created on Sun Sep  3 09:56:18 2017

@author: Andreas Franck a.franck@soton.ac.uk
"""

import visr
import rrl
import panning
import pml
import audiointerfaces as ai

from panning_comparison import PanningComparisonTrajectory

# Change to 'PyQt5' depending on your Python installation
#from PyQt4 import QtCore, QtGui

# Change to 'PyQt5' depending on your Python installation
#from PyQt5 import QtCore
#import PyQt5.QtWidgets as QtGui # Hacky renaming to get compatibility with the Qt4 namespaces

import numpy as np
import os
import sys

from vbap_renderer import VbapRenderer, FrequencyType
from allrad_panner import AllRadPanner

# Global variables

# Linux and MacOS
if sys.platform in ['linux', 'linux2','darwin']:
    # %% On Linux and MacOS X (with Jack)
    audioIfcName = 'Jack'
    optionalAudioCfg = """{ "clientname": "MultiRenderer",
                           "autoconnect" : "false",
                           "portconfig":
                               {
                               "capture":  [{ "basename":"inObj_", "externalport" : {} }],
                                            "playback": [{ "basename":"outBin_", "externalport" : {} }]
                                            }
                      }"""
elif sys.platform == 'win32':
    # %% PortAudio on Windows
    optionalAudioCfg = """{ "hostapi": "ASIO" }"""
    audioIfcName = 'PortAudio'
else:
    print( "Unsupported platform." )

samplingFrequency = 48000
blockSize = 1024
setup = 'Salford'
# setup = 'Southampton'


# List of rendering configurations
if setup == 'Southampton':

    from PyQt4 import QtCore, QtGui

    rendererConfigs = [
            { "className":VbapRenderer, "name":"VbapPlain",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner":False } },
            { "className":VbapRenderer, "name":"VbipPlain",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner":False } },
            { "className":VbapRenderer, "name":"DualPlain",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.Dual, "normalisationNorm": [1,2], "l2Panner":False } },
            { "className":VbapRenderer, "name":"VbapDownMix",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0_mpeg-h_downmix.xml"),
                        "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner":False } },
            { "className":VbapRenderer, "name":"VbipDownMix",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0_mpeg-h_downmix.xml"),
                        "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner":False } },
            { "className":VbapRenderer, "name":"DualDownMix",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0_mpeg-h_downmix.xml"),
                        "bandType": FrequencyType.Dual, "normalisationNorm": [1,2], "l2Panner":False } },
            { "className":VbapRenderer, "name":"VbapL2",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner": True } },
            { "className":VbapRenderer, "name":"VbipL2",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner": True } },
            { "className":VbapRenderer, "name":"DualL2",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.Dual, "normalisationNorm": [1, 2], "l2Panner":True } },
            { "className":AllRadPanner, "name": "AllRAD", "kwArgs": { "realArrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
             "regularArrayConfig": panning.LoudspeakerArray("../data/t-design_order11_n70.xml"), "hoaOrder": 2 }  }
            ]

    configBasePath = '../data/'
    renderConfigName = 'bs2051-4+5+0.xml'
    channelRouting = list(range(0,9))

elif setup == 'Salford':

    from PyQt5 import QtCore
    import PyQt5.QtWidgets as QtGui # Hacky renaming to get compatibility with the Qt4 namespaces

    rendererConfigs = [
        { "className":VbapRenderer, "name":"VbapPlain",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner":False } },
        { "className":VbapRenderer, "name":"VbipPlain",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner":False } },
        { "className":VbapRenderer, "name":"DualPlain",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.Dual, "normalisationNorm": [1,2], "l2Panner":False } },
        { "className":VbapRenderer, "name":"VbapDownMix",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0_mpeg-h_downmix.xml"),
                    "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner":False } },
        { "className":VbapRenderer, "name":"VbipDownMix",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0_mpeg-h_downmix.xml"),
                    "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner":False } },
        { "className":VbapRenderer, "name":"DualDownMix",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0_mpeg-h_downmix.xml"),
                    "bandType": FrequencyType.Dual, "normalisationNorm": [1,2], "l2Panner":False } },
        { "className":VbapRenderer, "name":"VbapL2",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner": True } },
        { "className":VbapRenderer, "name":"VbipL2",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner": True } },
        { "className":VbapRenderer, "name":"DualL2",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.Dual, "normalisationNorm": [1, 2], "l2Panner":True } },
        { "className":AllRadPanner, "name": "AllRAD", "kwArgs": { "realArrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
         "regularArrayConfig": panning.LoudspeakerArray("../data/t-design_order11_n70.xml"), "hoaOrder": 2 }  }
        ]

    configBasePath = '../data/'
    renderConfigName = 'SalfordAudioBooth-4+5+0.xml'
    channelRouting = [0,1,3,5,7,12,13,14,15]

trajectoryGenerator = True

# Create files with full paths
configPath = os.path.join( configBasePath, renderConfigName )

# testSignal = None # Use a soundcard input
testSignal = os.path.join( configBasePath, 'pink_noise_1ch_48000Hz.wav' )

lspConfig = panning.LoudspeakerArray( configPath )
numberOfInputs = 1
numberOfOutputs = lspConfig.numberOfRegularLoudspeakers

numRenderers = 5

if trajectoryGenerator:
    updatePeriod = 4096
    cycleDuration = 8.0

    numPos = int(np.ceil( cycleDuration*samplingFrequency / updatePeriod ))

    # Create the same azimuthal movement at different elevations
    azimuths = np.tile( np.arange( 0, 2.0*np.pi, 2.0*np.pi/numPos ), [3,1] )

    elevations = np.tile( [0.1, 0.2,0.3], [ numPos, 1] ).T
    radius = 1.0
    # Could be replaced by a sph2cart library function.
    positions = radius * np.stack( (np.cos(azimuths)*np.cos(elevations),
                         np.sin(azimuths)*np.cos(elevations),
                         np.sin(elevations))
                         )

    numTrajectories = positions.shape[0]

# Optional gain and delay adjustments

# Gains in linear scale, use 'None' for no gain adjustment
# gainAdjustment = None
gainAdjustment = np.asarray([ 1.0, 0.9, 0.8, 0.8, 0.9, 0.7, 0.7, 0.7, 0.7  ])

# Delay adjustment in second (here: miliseconds plus scaling)
delayAdjustment = None
delayAdjustment = np.asarray([ 0.0, 1.2, 2.0, 2.0, 2.0, 1.0, 1.0, 1.0 ]) * 1.0e-3


class PanningComparisonGui( QtGui.QWidget ): # QtGui.QMainWindow
    def __init__( self, rendererNames ):
        # Set up the UI
        super().__init__( None )
        self.audioRunning = False
        self.setGeometry( 0, 0, 800, 600 )

        mainLayout = QtGui.QVBoxLayout()

#        topLayout = QtGui.QHBoxLayout()
#
#        self.startButton = QtGui.QPushButton( "Start Audio" )
#        self.stopButton = QtGui.QPushButton( "Stop Audio" )
#        self.startButton.clicked.connect( self.startAudio )
#        self.stopButton.clicked.connect( self.stopAudio )
#        topLayout.addWidget( self.startButton )
#        topLayout.addWidget( self.stopButton )
#
#        mainLayout.addLayout( topLayout )

        trajectorySelectLayout = QtGui.QHBoxLayout()
        self.trajectorySelect = QtGui.QComboBox()
        for trajIdx in range(numTrajectories ):
            self.trajectorySelect.addItem( "Trajectory %d" % trajIdx )
        self.trajectorySelect.currentIndexChanged[int].connect( self.trajectorySelected )

        trajectorySelectLayout.addWidget( self.trajectorySelect )
        mainLayout.addLayout( trajectorySelectLayout )

        rendererLayout = QtGui.QGridLayout()

        self.sliders = []
        self.buttons = []

        self.buttonGroup = QtGui.QButtonGroup()
        self.buttonGroup.setExclusive( True )

        rendererLayout.setColumnStretch( 0, 1 )
        qualityLabels = ['Excellent', 'Good', 'Fair', 'Poor', 'Bad' ]
        for idx, labelText in enumerate( qualityLabels):
            labelL = QtGui.QLabel( labelText )
            labelL.setAlignment( QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            rendererLayout.addWidget( labelL, idx, 0 )

            labelR = QtGui.QLabel( labelText )
            labelR.setAlignment( QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter )
            rendererLayout.addWidget( labelR, idx, len(rendererNames)+1 )

        for rIdx, rName  in enumerate( rendererNames ):
            rendererLayout.setColumnStretch( rIdx+1, 1 )
            sl = QtGui.QSlider()
            sl.setMaximum(100)
            sl.setMinimum(0)
            sl.setOrientation( QtCore.Qt.Vertical )
            sl.setTickPosition( QtGui.QSlider.TicksBothSides )
            sl.setTickInterval( 20 )
            sl.setValue( 80 )

            self.sliders.append( sl )
            rendererLayout.addWidget( sl, 0, rIdx+1, 5, 1, QtCore.Qt.AlignHCenter )

            pb = QtGui.QPushButton()
            pb.setText( rName )
            pb.setCheckable( True )
            pb.setStyleSheet( "QPushButton:checked { background-color: green }" )
#            pb.setStyleSheet("QPushButton { background-color: blue }"
#                             "QPushButton:pressed { background-color: red }" )
            rendererLayout.addWidget( pb, 5, rIdx+1)

            self.buttons.append( pb )
            self.buttonGroup.addButton( pb, rIdx )

        self.buttonGroup.buttonClicked[int].connect( self.rendererButtonPressed )

        mainLayout.addLayout( rendererLayout )

        self.setLayout( mainLayout )

        # %% Setup the DSP part

        self.context = visr.SignalFlowContext( period = blockSize,
                                               samplingFrequency = samplingFrequency )
        self.component = PanningComparisonTrajectory( self.context, 'MultiRenderer', None,
                                           rendererConfigs=rendererConfigs,
                                           numberOfInputs=numberOfInputs,
                                           numberOfOutputs=numberOfOutputs,
                                           controlDataType=pml.Integer,
                                           trajectoryPositions=positions,
                                           trajectoryUpdateSamples = updatePeriod,
                                           testSignal = testSignal,
                                           useDummyInput=True,
                                           delayAdjust = delayAdjustment,
                                           gainAdjust = gainAdjustment,
                                           outputChannelRouting = channelRouting,
                                           trajectoryControlInputType = pml.Integer, # Activates the control input
                                                     # different trajectories.
                                           initialTrajectoryIndex = 0
                                           )

        res, msg = rrl.checkConnectionIntegrity( self.component )
        if not res:
            raise RuntimeError( msg )

        self.flow = rrl.AudioSignalFlow( self.component )

        audioOptions = ai.AudioInterface.Configuration( self.flow.numberOfCaptureChannels,
                                                       self.flow.numberOfPlaybackChannels,
                                                       samplingFrequency,
                                                       blockSize )

        self.audioInterface = ai.AudioInterfaceFactory.create( audioIfcName,
                                                              audioOptions,
                                                              optionalAudioCfg )

        self.switchPort = self.flow.parameterReceivePort( 'switch' )
        self.trajectoryPort = self.flow.parameterReceivePort( 'trajectoryControlInput' )


    def startAudio( self ):
        if self.audioRunning:
            return
        self.audioInterface.registerCallback( self.flow )
        self.audioInterface.start()
        self.audioRunning = True
        return
    def stopAudio( self ):
        if not self.audioRunning:
            return
        self.audioInterface.stop()
        self.audioInterface.unregisterCallback()
        self.audioRunning = False
        return


    def rendererButtonPressed( self, idx  ):
        print( "Renderer selection button pressed with id %d" % idx )
        self.switchPort.enqueue( pml.Integer( idx ) )

    def trajectorySelected( self, index ):
        print( "Trajectory %d selected." % index )
        self.trajectoryPort.enqueue( pml.Integer( index ) )


app = QtGui.QApplication( sys.argv )

rendererNames = [ i["name"] for i in rendererConfigs]

mainWnd = PanningComparisonGui( rendererNames )

mainWnd.show()
mainWnd.startAudio()

appReturnValue = app.exec_()

print( "Returned from app.exec_()" )

del mainWnd
del app
